DROP SCHEMA IF EXISTS `SuperGeeks`;
CREATE SCHEMA `SuperGeeks`;
USE `SuperGeeks`;
DROP TABLE IF EXISTS `Parents`;
CREATE TABLE `Parents`(
parentID varchar(40) not null unique,
primary key (parentID),
fname varchar(35),
lname varchar(35),
email varchar(320) unique,
-- maximum size of an email address should be able to store absolutely all possible email addresses
-- unable to view payment info from any select functions it is also encrypted for customer safety
paymentinfo varchar(50),
subscribed bool,
phonenum int(10));
DROP TABLE IF EXISTS `Students`;
CREATE TABLE `Students`(
studentID varchar(40) not null unique,
studentCode int key auto_increment,
fname varchar(35),
lname varchar(35),
-- The UK Government Data Standards Catalogue reccomends 35 characters for each the first name and last name
bday date,
dayjoined date DEFAULT CURDATE() ,
parentsID varchar(40),
groupID varchar(40),
FOREIGN KEY (parentsID) REFERENCES Parents(parentID)
-- parent ID is stored in student rather than the parents table so that one parent can have many children
-- and not one child can have many parents. A child may have multiple parents irl but they would share an
-- account
);
ALTER TABLE Students auto_increment = 30001;
DROP TABLE IF EXISTS `Volunteers`;
CREATE TABLE `Volunteers`(
	VolunteerID varchar(40) not null unique,
    VolunteerCode int auto_increment,
    primary key (volunteerCode,VolunteerID),
    fname varchar(35),
    lname varchar(35),
    hoursworked int(8) DEFAULT 0,
    DBSdue date,
    SGdue date,
    emergencycontact varchar(20),
    email varchar(320)
	);
alter table Volunteers auto_increment = 1001;
DROP TABLE IF EXISTS `Sessions`;
CREATE TABLE `Sessions`(
	sessionID varchar(40) not null unique,
    primary key (sessionID),
    leaderID varchar(40),
    location varchar(35),
    duration int(10),
		occuredOn date,
    FOREIGN KEY (leaderID) REFERENCES Volunteers(VolunteerID),
    maxAttendees int(10),
    groupid varchar(40),
		reviewOfProgression varchar(400),
		reviewOfstaffSafety varchar(400),
		Nohelpers int(10));

DROP TABLE IF EXISTS `Badge`;
CREATE TABLE `Badge`(
	BadgeID varchar(40) not null unique,
    primary key(BadgeID),
    badgeName varchar(35),
    badgeDesc varchar(255),
		ageGroup int(1)
	);
		DROP TABLE IF EXISTS `WaitList`;
		CREATE TABLE `WaitList`(

			studentID varchar(40) unique,
			SessionID varchar(40),
			dateJoined date,
			dateRemoved date,
	 		ageGroup int(1),
			FOREIGN KEY (studentID) REFERENCES Students(studentID),
			FOREIGN KEY (SessionID) REFERENCES Sessions(SessionID));



DROP TABLE IF EXISTS `BadgeAwarded`;
CREATE TABLE `BadgeAwarded`(
	awardedto varchar(40),
	evidence varchar(1000),
    awardedby varchar(40),
    Badgeawarded varchar(40),
    dateAwarded date,
		isAwarded boolean,
		-- isAwarded is to review applications
	FOREIGN KEY (awardedto) REFERENCES Students(studentID),
	FOREIGN KEY (awardedby) REFERENCES Volunteers(VolunteerID),
	FOREIGN KEY (Badgeawarded) REFERENCES Badge(BadgeID)
    );
-- DROP TABLE IF EXISTS `Badgereqs`;
CREATE TABLE `Badgereqs`(
    mustbecompleted varchar(40),
    tocomplete varchar(40),
    FOREIGN KEY (mustbecompleted) REFERENCES Badge(BadgeID),
    FOREIGN KEY (tocomplete) REFERENCES Badge(BadgeID)
);
DROP TABLE IF EXISTS `StudentAttendance`;
CREATE TABLE `StudentAttendance`(
	studentID varchar(40),
    sessionID varchar(40),
	FOREIGN KEY (studentID) REFERENCES Students(studentID),
	FOREIGN KEY (sessionID) REFERENCES Sessions(SessionID)
);
DROP TABLE IF EXISTS `VolunteerAttendance`;
CREATE TABLE `VolunteerAttendance`(
	volunteerID varchar(40),
    sessionID varchar(40),
	FOREIGN KEY (VolunteerID) REFERENCES Volunteers(volunteerID),
	FOREIGN KEY (sessionID) REFERENCES Sessions(SessionID)
);



-- procedures 
drop procedure if exists addParent;
delimiter $$
CREATE PROCEDURE addParent(
    	IN fname varchar(35),
	IN lname varchar(35),
	IN email varchar(320),
	IN passwerd varchar(70),
	IN phonenum int(10))

BEGIN
    DECLARE EXIT HANDLER FOR 1062
    BEGIN
 	SELECT 'User already exists' as ERROR ;
    END;

--    if (locate("@",email) = 0
	-- 	Select "invalid email as Error";

        INSERT INTO Parents(parentID,fname,lname,email,phonenum) values
	(md5(concat(email,passwerd)),fname,lname,email,phonenum);
END$$addParent
DELIMITER ;
drop procedure if exists addVolunteer;
delimiter $$
CREATE PROCEDURE addVolunteer(
	IN fname varchar(35),
	IN lname varchar(35),
	IN email varchar(320),
	IN passwerd varchar(70),
	IN phonenum int(10))

BEGIN
    DECLARE EXIT HANDLER FOR 1062
    BEGIN
 	SELECT 'User already exists' as ERROR ;
    END;
		IF INSTR(email,"@") > 0 then

        INSERT INTO Volunteers(VolunteerID,fname,lname,email,emergencycontact) values
	(md5(concat(email,passwerd)),fname,lname,email,phonenum);

	end if;
END$$
DELIMITER ;

drop procedure if exists addBadgereq;
delimiter $$
CREATE PROCEDURE addBadgereq(
	IN youNeed varchar(40),
	IN  toGet varchar(40))
	-- I use this naming scheme as it just makes it easy to understand
	-- which badge is which in this relationship

BEGIN
    DECLARE EXIT HANDLER FOR 1452
    BEGIN
 	SELECT 'Badge not found' as ERROR ;
    END;

        INSERT INTO Badgereqs(mustbecompleted,tocomplete) values
	(youNeed,toGet) ;
END$$
DELIMITER ;

drop procedure if exists addBadge;
delimiter $$
CREATE PROCEDURE addBadge(
	IN badgeName varchar(40),
	IN badgeDesc varchar(255))

BEGIN
    DECLARE EXIT HANDLER FOR 1062
    BEGIN
 	SELECT ' Badge already exists' as ERROR ;
    END;

        INSERT INTO Badge(BadgeID,badgeName,badgeDesc) values
	(md5(badgeName),badgeName,badgeDesc);
END$$
DELIMITER ;

drop procedure if exists `applyToGroup`;
delimiter $$
CREATE PROCEDURE applyToGroup(
	IN studentID varchar(40),
	IN SessionID varchar(255),
	IN ageGroup int(1))
BEGIN
    DECLARE EXIT HANDLER FOR 1062
    BEGIN
 	SELECT 'This student is already on a wait list' as ERROR ;
    END;

        INSERT INTO WaitList(StudentID,SessionID,dateJoined,ageGroup) values
	(StudentID,SessionID,CURDATE(),ageGroup);
END$$
DELIMITER ;


drop procedure if exists addStudent;
delimiter $$
CREATE PROCEDURE addStudent(
	IN fname varchar(35),
	IN lname varchar(35),
    IN bday date,
    IN parentsID varchar(40),
	IN passwerd varchar(70))

BEGIN
    DECLARE EXIT HANDLER FOR 1062
    BEGIN
 	SELECT 'User already exists' as ERROR ;
    END;

        INSERT INTO Students(StudentID,fname,lname,bday,parentsID) values
	(md5(concat(fname,lname,passwerd)),fname,lname,bday,parentsID);

END$$
DELIMITER ;

drop procedure if exists viewStudent;
delimiter $$
CREATE PROCEDURE viewStudent(
	IN studentvID varchar(40))
BEGIN
	If LENGTH(studentvID) = 0
	then
 	SELECT s0.StudentID , s0.studentCode, s0.fname ,s0.lname,s0.bday,(select email from parents where parentID = s0.parentsID),(select phonenum from parents where parentID = s0.parentsID) from students as s0;
	else
	 	SELECT s0.StudentID , s0.studentCode, s0.fname ,s0.lname,s0.bday,(select email from parents where parentID = s0.parentsID),(select phonenum from parents where parentID = s0.parentsID) from students as s0 where studentvID=s0.studentID;
end if;
END$$
DELIMITER ;


drop procedure if exists viewVolunteer;
delimiter $$
CREATE PROCEDURE viewVolunteer(
	IN VolunteervID varchar(40))
BEGIN
	If LENGTH(volunteervID) = 0
	then
 	SELECT s0.VolunteerID , s0.volunteerCode, s0.fname ,s0.lname,s0.DBSdue,s0.SGdue,s0.emergencycontact,s0.email,(select sum(duration) from Sessions where sessionID in(select sessionID from VolunteerAttendance where volunteerID=s0.volunteerID)) from volunteers as s0;
	else
	 		SELECT s0.VolunteerID , s0.volunteerCode, s0.fname ,s0.lname,s0.DBSdue,s0.SGdue,s0.emergencycontact,s0.email,(select sum(duration) from Sessions where sessionID in(select sessionID from VolunteerAttendance where volunteerID=s0.volunteerID)) from volunteers as s0 where volunteerID=volunteervID;
end if;
END$$
DELIMITER ;

drop procedure if exists Studentattends;
delimiter $$
CREATE PROCEDURE Studentattends(
    IN studentID varchar(40),
		IN sessionID varchar(40))

BEGIN

        INSERT INTO StudentAttendance(StudentID,sessionID) values
	(studentID,sessionID);

END$$
DELIMITER ;

drop procedure if exists volunteerattends;
delimiter $$
CREATE PROCEDURE volunteerattends(
    IN volunteerID varchar(40),
		IN sessionID varchar(40))

BEGIN

        INSERT INTO volunteerAttendance(volunteerID,sessionID) values
	(volunteerID,sessionID);

END$$
DELIMITER ;

drop procedure if exists addSession;
delimiter $$
CREATE PROCEDURE addSession(
	IN location varchar(35),
	IN duration int(10),
	IN learderID varchar(40),
    IN OccuredOn date,
    IN groupID varchar(40))

BEGIN
		DECLARE EXIT HANDLER FOR 1062
    BEGIN
 	SELECT 'User already exists' as ERROR ;
    END;
        INSERT INTO Sessions(SessionID,location,duration,OccuredOn,groupID,leaderID) values
	(md5(concat(duration,groupID)),location,OccuredOn,groupID,leaderID);
END$$
DELIMITER ;


drop procedure if exists ReviewSession;
delimiter $$
CREATE PROCEDURE ReviewSession(
	IN SessionID varchar(40),
	IN reviewOfProgression varchar(400),
	IN reviewOfstaffSafety varchar(400))

BEGIN
        UPDATE Sessions set reviewOfstaffSafety=reviewOfstaffSafety,reviewOfProgression=reviewOfProgression where SessionID=SessionID;
END$$
DELIMITER ;

drop procedure if exists updateDBS;
delimiter $$
CREATE PROCEDURE updateDBS(
	IN VolunteerID varchar(40),
	IN DateOfDBSApproval date)

BEGIN
        UPDATE volunteers set DBSdue =date_add(DateOfDBSApproval, interval 3 year)  where VolunteerID=volunteerID;
END$$
DELIMITER ;

drop procedure if exists updateSG;
delimiter $$
CREATE PROCEDURE updateSG(
	IN VolunteerID varchar(40),
	IN DateOfSGApproval date)

BEGIN
        UPDATE volunteers set SGdue =date_add(DateOfSGApproval, interval 5 year)  where VolunteerID=volunteerID;
END$$
DELIMITER ;

drop procedure if exists ViewCheckExpiries;
delimiter $$
CREATE PROCEDURE ViewCheckExpiries()

BEGIN
        SELECT volunteerID,fname,lname,DBSdue,SGdue,
				if(date_sub(DBSdue, interval 6 month) > CURDATE()=1,
				 		"ALL GOOD",
						if(CURDATE()>DBSdue=1
							,concat("already Due ",fname," does not have a valid dbs check"),
							concat("needs redoing in the before",DBSdue))) as "dbs refiew" ,
				if(date_sub(SGdue, interval 6 month) > CURDATE()=1 ,
				"ALL GOOD",
				if(CURDATE()>DBSdue=1
					,concat("already Due ",fname," does not have a valid SG check")

							concat("needs redoing in the before",SGdue))) as "SuperGeeks training reviewz"
					from volunteers where date_sub(DBSdue, interval 6 month) < CURDATE()=1 or  date_sub(SGdue, interval 6 month) < CURDATE()=1;

END$$
DELIMITER ;


drop procedure if exists StudentBadgeApply;
delimiter $$
CREATE PROCEDURE StudentBadgeApply(
    IN studentID varchar(40),
		IN BadgeID varchar(40),
		IN badgeEvidence varchar(1000))

BEGIN

        INSERT INTO StudentAttendance(StudentID,sessionID) values
	(studentID,sessionID);

END$$
DELIMITER ;




call addParent("jeff","Jones","email@realemail.com","veryStrongPassw0rd",02358254);
call addStudent("bobby","jones",'2012-2-11','8e301d6e513165a9c0b6b9a2b10c8305',"EXTRASTRONGPASSWORD");
call addBadge("real badge","this is a totally real badge");
call addStudent("timothy","jones",'2002-2-11','8e301d6e513165a9c0b6b9a2b10c8305',"EXTREMELYSTRONGPASSWORD");
call addVolunteer("timothy","jones",'bob@bbc.co.uk','8e301d6e513165a9c0b6b9a2b10c8305',"0777008913");

call updateSG('419fd9859978a9bf79df4ff16c99c5f9',"2015-09-09");
call updateDBS('419fd9859978a9bf79df4ff16c99c5f9',"2021-09-09");

call viewCheckExpiries();

-- select * from volunteers;
